@@ -706,6 +706,7 @@ int master_loop(char **argv, char **environ) {
 			uwsgi_master_check_gateways_deadline();
 			uwsgi_master_check_mules_deadline();
 			uwsgi_master_check_spoolers_deadline();
+			uwsgi_master_check_crons_deadline();
 
 #ifdef __linux__
 #ifdef MADV_MERGEABLE
@@ -789,6 +790,7 @@ int master_loop(char **argv, char **environ) {
 			if (uwsgi_master_check_mules_death(diedpid)) continue;
 			if (uwsgi_master_check_gateways_death(diedpid)) continue;
 			if (uwsgi_master_check_daemons_death(diedpid)) continue;
+			if (uwsgi_master_check_cron_death(diedpid)) continue;
 		}
 
 
