@@ -56,6 +56,7 @@ struct uwsgi_router_cache_conf {
 
 	char *status_str;
 	int status;
+	char *no_offload;
 };
 
 // this is allocated for each transformation
@@ -189,6 +190,14 @@ static int uwsgi_routing_func_cache(struct wsgi_request *wsgi_req, struct uwsgi_
 			if (uwsgi_response_add_expires(wsgi_req, expires)) goto error;	
 		}
 		if (uwsgi_response_add_content_length(wsgi_req, valsize)) goto error;
+		if (wsgi_req->socket->can_offload && !ur->custom && !urcc->no_offload) {
+                	if (!uwsgi_offload_request_memory_do(wsgi_req, value, valsize)) {
+                        	wsgi_req->via = UWSGI_VIA_OFFLOAD;
+                       		wsgi_req->status = 202;
+                        	return UWSGI_ROUTE_BREAK;
+                	}
+		}
+
 		uwsgi_response_write_body_do(wsgi_req, value, valsize);
 		free(value);
 		if (ur->custom)
@@ -369,6 +378,7 @@ static int uwsgi_router_cache(struct uwsgi_route *ur, char *args) {
                         "content_encoding", &urcc->content_encoding,
                         "mime", &urcc->mime,
                         "name", &urcc->name,
+                        "no_offload", &urcc->no_offload,
                         NULL)) {
 			uwsgi_log("invalid route syntax: %s\n", args);
 			exit(1);
